/*
 Turnigy TGY-i6 RSSI LED Indicator
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 3/09/2015
 Last Modified: 3/09/2015
 
 Monitors the RSSI byte coming from the Turnigy TGY-i6 transmitter and light an LED when the error rate is high.
 
 Remember to change your fuse bits so the ATtiny85 uses 16MHz PLL and divide clock by 1
 avrdude -p ATtiny85 -c usbtiny -U lfuse:w:0xe1:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m 
 
 */
 
// ATtiny25/45/85 Pin map
//
//              +-\/-+
//  Reset PB5  1|o   |8  VCC
//        PB3  2|    |7  PB2 CLK (Clock)
//    LED PB4  3|    |6  PB1 
//        GND  4|    |5  PB0 GOIO (Data)
//              +----+

#define F_CPU 16000000 // 16 MHz

#define goioIn PB0
#define led PB4

#define RSSI_COMPARE_TO 45 // Change to suit your error rate needs (testing required)

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <math.h>
#include "setup.c"

uint8_t led_on = 0;

int main(void) {
	setup();
	
	// Blink LED at start
	PORTB |= (1<<led);
	_delay_ms(100);
	PORTB &= ~(1<<led);
	
	// Initialise USI properly, wait for GOIO pin high for a while
	uint8_t counter = 0;
	while (1) {
		if (PINB & (1<<goioIn)) {
			counter++;
			if (counter >= 30) {
				USICR = (1<<USIWM0) | (1<<USICS1); // Enable USI
				break;
			}
		}
		else {
			counter = 0;
		}
	}
	
	while(1) {
		while (!(USISR & (1<<USIOIF))); // Wait until new serial data received
		USISR |= (1<<USIOIF); // Clear received flag
		if (USIBR == 0xFE) {
			while (!(USISR & (1<<USIOIF)));
			USISR |= (1<<USIOIF);
			if (USIBR == 0x00) {
				while (!(USISR & (1<<USIOIF)));
				USISR |= (1<<USIOIF);
				
				// RSSI byte
				if (USIBR >= RSSI_COMPARE_TO) {
					OCR0A = USIBR;
					led_on = 1;
				}
				else {
					OCR0A = 0;
					led_on = 0;
				}
			}
		}
	}
}

// Timer overflow
ISR(TIM0_OVF_vect) {
	if (led_on == 1) {
		PORTB |= (1<<led);
	}
}

// Timer OCR0A compare match
ISR(TIMER0_COMPA_vect) {
	PORTB &= ~(1<<led);
}
